/**************************************************************************
*
* Filename:    PS3000Acon.c
*
* Copyright:   Pico Technology Limited 2010
*
* Author:      RPM & Peter Parfenov (2012)
*
* Description:
*   This is a console mode program that demonstrated how to use the
*   PicoScope 3000a series API. Now one collects data for averaging.
*
* Examples:
*    Collect a block of samples immediately
*    Collect a block of samples when a trigger event occurs
*    Collect a stream of data immediately
*    Collect a stream of data when a trigger event occurs
*    Set Signal Generator, using standard or custom signals
*
*	To build this application:
*			 Set up a project for a 32-bit console mode application
*			 Add this file to the project
*			 Add PS3000A.lib to the project
*			 Add ps3000aApi.h and picoStatus.h to the project
*			 Build the project
*
***************************************************************************/
#include <stdio.h>

/* Headers for Windows */
#ifdef _WIN32
#include "windows.h"
#include <conio.h>
#include <math.h>
#include "ps3000aApi.h"
#else
#include <sys/types.h>
#include <string.h>

#include <libps3000a-1.0/ps3000aApi.h>
#include "linux_utils.h"
#endif

#define PREF4 __stdcall


struct ps3000aSetting {
	int percTimeShift; // delay from the frame start in percent (unused)
	int nBufferSize;// The Buffer Size (default=1024)
	int nTriggerLevel;// The Trigger Level, mV (default=-100)
	char bEdgeSelect;// Select of signal edge - fall/rise (default=PS3000A_FALLING)
	int nWaveForms;// The Number of wave forms (default=25000)
	unsigned long timeBase;// Time base scale,  0 - 2ns, 1 - 4ns, 2 - 8ns, 3 - 16 ns, 4 - 32ns (default=1).
	// setting for the channels
	short channelRange;// The Channel range, PS3000A_50MV, ...100,200,500..., PS3000A_1V, ...5,10...PS3000A_20V (default=PS3000A_500MV) 
	BOOL channelDCcoupled;// The Channel mode, DC/AC=TRUE/FALSE (default=TRUE)
	BOOL switchChannelA;// Channel A switch, on/off=TRUE/FALSE (default=TRUE)
	BOOL switchChannelB;// Channel B switch, on/off=TRUE/FALSE (default=FALSE)
};

struct	ps3000aSetting mySetting;

unsigned long timebase;

// variables for a tune of timescale
short       oversample = 1;
BOOL      scaleVoltages = TRUE;

typedef struct
{
	short DCcoupled;
	short range;
	short enabled;
}CHANNEL_SETTINGS;



typedef struct tTriggerDirections
{
	PS3000A_THRESHOLD_DIRECTION channelA;
	PS3000A_THRESHOLD_DIRECTION channelB;
	PS3000A_THRESHOLD_DIRECTION channelC;
	PS3000A_THRESHOLD_DIRECTION channelD;
	PS3000A_THRESHOLD_DIRECTION ext;
	PS3000A_THRESHOLD_DIRECTION aux;
}TRIGGER_DIRECTIONS;

typedef struct tPwq
{
	PS3000A_PWQ_CONDITIONS * conditions;
	short nConditions;
	PS3000A_THRESHOLD_DIRECTION direction;
	unsigned long lower;
	unsigned long upper;
	PS3000A_PULSE_WIDTH_TYPE type;
}PWQ;

typedef struct
{
	short handle;
	PS3000A_RANGE			firstRange;
	PS3000A_RANGE			lastRange;
	short					channelCount;
	short					maxValue;
	CHANNEL_SETTINGS	channelSettings [PS3000A_MAX_CHANNELS];
}UNIT;


unsigned short inputRanges [PS3000A_MAX_RANGES] = {	10, 
													20, 
													50,
													100,
													200,
													500,
													1000,
													2000,
													5000,
													10000,
													20000};
BOOL     			g_ready = FALSE;
long long 		g_times [PS3000A_MAX_CHANNELS];
short     		g_timeUnit;
long      		g_sampleCount;
unsigned long	g_startIndex;
short			g_autoStopped;
short			g_trig = 0;
unsigned long	g_trigAt = 0;

char cFileName[30]="Nrapidblock_average.txt"; // The name of the output file for averaging data.
char cSettingFile[20]="piqo_setting.dat"; // The name of the setting file.

/****************************************************************************
* Callback
* used by PS3000A data streaimng collection calls, on receipt of data.
* used to set global flags etc checked by user routines
****************************************************************************/
void PREF4 CallBackStreaming(	short handle,
	long noOfSamples,
	unsigned long	startIndex,
	short overflow,
	unsigned long triggerAt,
	short triggered,
	short autoStop,
	void	*pParameter)
{
	// used for streaming
	g_sampleCount = noOfSamples;
	g_startIndex	= startIndex;
	g_autoStopped		= autoStop;

	// flag to say done reading data
	g_ready = TRUE;

	// flags to show if & where a trigger has occurred
	g_trig = triggered;
	g_trigAt = triggerAt;
}

/****************************************************************************
* Callback
* used by PS3000A data block collection calls, on receipt of data.
* used to set global flags etc checked by user routines
****************************************************************************/
void PREF4 CallBackBlock( short handle, PICO_STATUS status, void * pParameter)
{
	if (status != PICO_CANCELLED)
		g_ready = TRUE;
}

/****************************************************************************
* SetDefaults - restore default settings
****************************************************************************/
void SetDefaults(UNIT * unit) 
{
	PICO_STATUS status;
	int i;

	status = ps3000aSetEts(unit->handle, PS3000A_ETS_OFF, 0, 0, NULL); // Turn off ETS

	for (i = 0; i < unit->channelCount; i++) // reset channels to most recent settings
	{
		status = ps3000aSetChannel(unit->handle, PS3000A_CHANNEL_A + i,
			unit->channelSettings[PS3000A_CHANNEL_A + i].enabled,
			unit->channelSettings[PS3000A_CHANNEL_A + i].DCcoupled,
			unit->channelSettings[PS3000A_CHANNEL_A + i].range, 0);
	}
}

/****************************************************************************
* adc_to_mv
*
* Convert an 16-bit ADC count into millivolts
****************************************************************************/

int adc_to_mv(long raw, int ch, UNIT * unit)
{
	return (raw * inputRanges[ch]) / unit->maxValue;
}

double dAdc_to_mv(double dRaw, int ch, UNIT * unit)
{
	return (dRaw * (double)inputRanges[ch]) / unit->maxValue;
}

/****************************************************************************
* mv_to_adc
*
* Convert a millivolt value into a 16-bit ADC count
*
*  (useful for setting trigger thresholds)
****************************************************************************/
short mv_to_adc(short mv, short ch, UNIT * unit)
{
	return (mv * unit->maxValue) / inputRanges[ch];
}

/****************************************************************************
* BlockDataHandler
* - Used by all block data routines
* - acquires data (user sets trigger mode before calling), displays 10 items
*   and saves all to block.txt
* Input :
* - unit : the unit to use.
* - text : the text to display before the display of data slice
* - offset : the offset into the data buffer to start the display's slice.
****************************************************************************/
void BlockDataHandler(UNIT * unit, char * text, int offset)
{
	int i, j;
	long timeInterval;
	long sampleCount= mySetting.nBufferSize;
	FILE * fp = NULL;
	long maxSamples;
	short * buffers[PS3000A_MAX_CHANNEL_BUFFERS];
	long timeIndisposed;
	PICO_STATUS status;

	for (i = 0; i < unit->channelCount; i++) 
	{
		buffers[i * 2] = (short*)malloc(sampleCount * sizeof(short));
		buffers[i * 2 + 1] = (short*)malloc(sampleCount * sizeof(short));
		status = ps3000aSetDataBuffers(unit->handle, (short)i, buffers[i * 2], buffers[i * 2 + 1], sampleCount, 0, PS3000A_RATIO_MODE_NONE);

//		printf("BlockDataHandler:ps3000aSetDataBuffers(channel %d) ------ 0x%08lx \n", i, status);
	}

	/*  find the maximum number of samples, the time interval (in timeUnits),
	*		 the most suitable time units, and the maximum oversample at the current timebase*/
	while (ps3000aGetTimebase(unit->handle, timebase, sampleCount, &timeInterval, oversample, &maxSamples, 0))
	{
		timebase++;
	}

	printf("\nTimebase: %lu  SampleInterval: %ldns  oversample: %hd\n", timebase, timeInterval, oversample);

	/* Start it collecting, then wait for completion*/
	g_ready = FALSE;
	status = ps3000aRunBlock(unit->handle, 0, sampleCount, timebase, oversample,	&timeIndisposed, 0, CallBackBlock, NULL);

//	printf("BlockDataHandler:ps3000aRunBlock ------ 0x%08lx \n", status);
	printf("Waiting for trigger...Press a key to abort\n");

	while (!g_ready && !_kbhit())
	{
		Sleep(0);
	}

	status = ps3000aStop(unit->handle);
//	printf("BlockDataHandler:ps3000aStop ------ 0x%08lx \n", status);

	if(g_ready) 
	{
		status = ps3000aGetValues(unit->handle, 0, (unsigned long*) &sampleCount, 1, PS3000A_RATIO_MODE_NONE, 0, NULL);
//		printf("BlockDataHandler:ps3000aGetValues ------ 0x%08lx \n", status);

		/* Print out the first 10 readings, converting the readings to mV if required */
		printf("%s\n",text);
		printf("Value (%s)\n", ( scaleVoltages ) ? ("mV") : ("ADC Counts"));

		for (j = 0; j < unit->channelCount; j++) 
		{
			printf("Channel%c:    ", 'A' + j);
		}
		printf("\n");

		for (i = offset; i < offset+10; i++) 
		{
			for (j = 0; j < unit->channelCount; j++) 
			{
				if (unit->channelSettings[j].enabled) 
				{
					printf("  %6d     ", scaleVoltages ? 
						adc_to_mv(buffers[j * 2][i], unit->channelSettings[PS3000A_CHANNEL_A + j].range, unit)	// If scaleVoltages, print mV value
						: buffers[j * 2][i]);																	// else print ADC Count
				}
			}
			printf("\n");
		}

		sampleCount = min(sampleCount, mySetting.nBufferSize);

		fopen_s(&fp, "block.txt", "w");
		if (fp != NULL)
		{
			fprintf(fp,"Results shown for CollectBlockTriggered  of the %d Channels\n\n",unit->channelCount);
			fprintf(fp, "Time Voltage\n");
			fprintf(	fp,	"ns		mV\n");
			for (i = 0; i < sampleCount; i++) 
			{
				fprintf(fp, "%5lld   ", g_times[0] + (long long)(i * timeInterval));
				fprintf(fp, "%lf     \n",	dAdc_to_mv((double)buffers[0][i], unit->channelSettings[PS3000A_CHANNEL_A].range, unit));
//				fprintf(fp, "%+6d",	adc_to_mv(buffers[0][i], unit->channelSettings[PS3000A_CHANNEL_A].range, unit));
						//fprintf(	fp, "%6d = %+6dmV  \n",	buffers[0][i],
						//	adc_to_mv(buffers[0][i], unit->channelSettings[PS3000A_CHANNEL_A].range, unit));
			}
		}
		else
			printf(	"Cannot open the file block.txt for writing.\n"
			"Please ensure that you have permission to access.\n");
	} 
	else 
	{
		printf("data collection aborted\n");
		_getch();
	}

	if (fp != NULL)
		fclose(fp);

	for (i = 0; i < unit->channelCount * 2; i++) 
	{
		free(buffers[i]);
	}
}


PICO_STATUS SetTrigger(	short handle,
						struct tPS3000ATriggerChannelProperties * channelProperties,
						short nChannelProperties,
						struct tPS3000ATriggerConditions * triggerConditions,
						short nTriggerConditions,
						TRIGGER_DIRECTIONS * directions,
						struct tPwq * pwq,
						unsigned long delay,
						short auxOutputEnabled,
						long autoTriggerMs)
{
	PICO_STATUS status;

	if ((status = ps3000aSetTriggerChannelProperties(handle,
													channelProperties,
													nChannelProperties,
													auxOutputEnabled,
													autoTriggerMs)) != PICO_OK) 
	{
		printf("SetTrigger:ps3000aSetTriggerChannelProperties ------ Ox%8lx \n", status);
		return status;
	}

	if ((status = ps3000aSetTriggerChannelConditions(handle, triggerConditions, nTriggerConditions)) != PICO_OK) 
	{
		printf("SetTrigger:ps3000aSetTriggerChannelConditions ------ 0x%8lx \n", status);
		return status;
	}

	if ((status = ps3000aSetTriggerChannelDirections(handle,
				directions->channelA,
				directions->channelB,
				directions->channelC,
				directions->channelD,
				directions->ext,
				directions->aux)) != PICO_OK) 
	{
		printf("SetTrigger:ps3000aSetTriggerChannelDirections ------ 0x%08lx \n", status);
		return status;
	}

	if ((status = ps3000aSetTriggerDelay(handle, delay)) != PICO_OK) 
	{
		printf("SetTrigger:ps3000aSetTriggerDelay ------ 0x%08lx \n", status);
		return status;
	}

	if((status = ps3000aSetPulseWidthQualifier(handle, 
				pwq->conditions,
				pwq->nConditions, 
				pwq->direction,
				pwq->lower, 
				pwq->upper, 
				pwq->type)) != PICO_OK)
	{
		printf("SetTrigger:ps3000aSetPulseWidthQualifier ------ 0x%08lx \n", status);
		return status;
	}

	return status;
}


/****************************************************************************
* CollectBlockTriggered
*  this function demonstrates how to collect a single block of data from the
*  unit, when a trigger event occurs.
****************************************************************************/
void CollectBlockTriggered(UNIT * unit)
{
	short	triggerVoltage = mv_to_adc(mySetting.nTriggerLevel, unit->channelSettings[PS3000A_CHANNEL_A].range, unit);

	struct tPS3000ATriggerChannelProperties sourceDetails = {	triggerVoltage,
																256 * 10,
																triggerVoltage,
																256 * 10,
																PS3000A_EXTERNAL, // There was "PS3000A_CHANNEL_A" in original file, but we use External input to synchronize.
																PS3000A_LEVEL};

	struct tPS3000ATriggerConditions conditions = {	PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_TRUE,
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_DONT_CARE};

	struct tPwq pulseWidth;

	struct tTriggerDirections directions = {	PS3000A_NONE,
												PS3000A_NONE,
												PS3000A_NONE,
												PS3000A_NONE,
												mySetting.bEdgeSelect,
												PS3000A_NONE };

	memset(&pulseWidth, 0, sizeof(struct tPwq));

	printf("Collect block triggered...\n");
	printf("Collects when value rises past %d", scaleVoltages?
		adc_to_mv(sourceDetails.thresholdUpper, unit->channelSettings[PS3000A_CHANNEL_A].range, unit)	// If there set scaleVoltages, then print value in mV.
		: sourceDetails.thresholdUpper);																// else print ADC Count
	printf(scaleVoltages?"mV\n" : "ADC Counts\n");

	printf("Press a key to start...\n");
	_getch();

	SetDefaults(unit);

	SetTrigger(unit->handle, &sourceDetails, 1, &conditions, 1, &directions, &pulseWidth, 0, 0, 0);

	BlockDataHandler(unit, "BlockDataHandler has ended, Ten readings after trigger\n", 0);
}

/****************************************************************************
* CollectRapidBlock
*  this function demonstrated how to collect a set of captures using 
*  rapid block mode. 
* Now one collects 25000 captures (frames) and average the frames into one frame.
****************************************************************************/
void CollectRapidBlock(UNIT * unit)
{
	unsigned short nCaptures;
	unsigned long nMaxSamples, nSamples = mySetting.nBufferSize;
	FILE * fp = NULL;
	long timeIndisposed;
	short capture, channel;
	short ***rapidBuffers;
	short *overflow;
	PICO_STATUS status;
	unsigned short i;
	unsigned long nCompletedCaptures;

	short	triggerVoltage = mv_to_adc(mySetting.nTriggerLevel, unit->channelSettings[PS3000A_CHANNEL_A].range, unit);

	// PS3000A_TRIGGER_CHANNEL_PROPERTIES structure, see the manual, p.82
	struct tPS3000ATriggerChannelProperties sourceDetails = {	triggerVoltage,
																256 * 10,
																triggerVoltage,
																256 * 10,
																PS3000A_EXTERNAL,
																PS3000A_LEVEL};

	// PS3000A_TRIGGER_CONDITIONS structure, see the manual,  p. 79. 
	struct tPS3000ATriggerConditions conditions = {	PS3000A_CONDITION_DONT_CARE, // It is channelA, there was "PS3000A_CONDITION_TRUE"
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_TRUE, // this is External input
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_DONT_CARE};

	struct tPwq pulseWidth;

	struct tTriggerDirections directions = {	PS3000A_NONE, // It is channelA, there was PS3000A_RISING
												PS3000A_NONE,
												PS3000A_NONE,
												PS3000A_NONE,
												mySetting.bEdgeSelect, //this is External input
												PS3000A_NONE };




	// These for ps3000aGetTimebase()
	long sampleCount; 
	long timeInterval;
	long maxSamples;

	sampleCount= nSamples;
	while (ps3000aGetTimebase(unit->handle, timebase, sampleCount, &timeInterval, oversample, &maxSamples, 0))
	{
		timebase++;
	}



	memset(&pulseWidth, 0, sizeof(struct tPwq));

	printf("Collect rapid block triggered...\n");
	printf("Collects when value rises past %d",	scaleVoltages?
		adc_to_mv(sourceDetails.thresholdUpper, unit->channelSettings[PS3000A_CHANNEL_A].range, unit)	// If there is scaleVoltages setting, then print value in mV
		: sourceDetails.thresholdUpper);																// else print ADC Count
	printf(scaleVoltages?"mV\n" : "ADC Counts\n");
	printf("Press any key to abort\n");

	// Set the number of captures. 
	// For our PS300A oscillograph internal memory the maximum is 25000 frames at 1024 points per frame.
	nCaptures = mySetting.nWaveForms; 

	SetDefaults(unit);
	
	// Trigger enabled
	if ((status = SetTrigger(unit->handle, &sourceDetails, 1, &conditions, 1, &directions, &pulseWidth, 0, 0, 0)) != PICO_OK) 
	{
		printf("CollectRapidBlock:SetTrigger ------ Ox%8lx \n", status);
		return;
	}

	//Segment the memory
	if ((status = ps3000aMemorySegments(unit->handle, nCaptures, &nMaxSamples)) != PICO_OK) 
	{
		printf("CollectRapidBlock:MemorySegments ------ Ox%8lx \n", status);
		return;
	}

	//Set the number of captures
	if ((status = ps3000aSetNoOfCaptures(unit->handle, nCaptures)) != PICO_OK) 
	{
		printf("CollectRapidBlock:SetNoOfCaptures ------ Ox%8lx \n", status);
		return;
	}

	//Run
	if ((ps3000aRunBlock(unit->handle, 0, nSamples, timebase, 1, &timeIndisposed, 0, CallBackBlock, NULL)) != PICO_OK) 
	{
		printf("CollectRapidBlock:RunBlock ------ Ox%8lx \n", status);
		return;
	}


	g_ready = FALSE; 
	printf("Waiting for trigger...\n");
	while(!g_ready && !_kbhit())
	{
		Sleep(0);
	}

	if(!g_ready)
	{
		_getch();
		status = ps3000aStop(unit->handle);
		status = ps3000aGetNoOfCaptures(unit->handle, &nCompletedCaptures);
		printf("Rapid capture aborted. %lu complete blocks were captured\n", nCompletedCaptures);
		printf("\nPress any key...\n\n");
		_getch();

		if(nCompletedCaptures == 0)
			return;

		nCaptures = (unsigned short)nCompletedCaptures; // The number of the blocks which were captured.
	}

	//Allocate memory
	rapidBuffers = calloc(unit->channelCount, sizeof(short*));
	overflow = calloc(unit->channelCount * nCaptures, sizeof(short));

	for (channel = 0; channel < unit->channelCount; channel++) 
	{
		rapidBuffers[channel] = calloc(nCaptures, sizeof(short*));
	}

	for (channel = 0; channel < unit->channelCount; channel++) 
	{	
		if(unit->channelSettings[channel].enabled)
		{
			for (capture = 0; capture < nCaptures; capture++) 
			{
				rapidBuffers[channel][capture] = calloc(nSamples, sizeof(short));
			}
		}
	}

	printf("Collecting data...\n");
	for (channel = 0; channel < unit->channelCount; channel++) 
	{
		if(unit->channelSettings[channel].enabled)
		{
			for (capture = 0; capture < nCaptures; capture++) 
			{
				status = ps3000aSetDataBuffer(unit->handle, channel, rapidBuffers[channel][capture], nSamples, capture, PS3000A_RATIO_MODE_NONE);
			}
		}
	}

	//Get data
	status = ps3000aGetValuesBulk(unit->handle, &nSamples, 0, nCaptures - 1, 1, PS3000A_RATIO_MODE_NONE, overflow);

	//Stop
	status = ps3000aStop(unit->handle);

// Averaging procedure
	printf("Averaged %hu captures      \n", nCaptures);
	// Averaging of the ADC points, and then convert them to mV.
		fopen_s(&fp, "rapidblock_average.txt", "w");
		if (fp != NULL)
		{
			double dAverage;
			long iSumm;
			double dAverageVoltage;

			fprintf(fp,"Results for CollectRapidBlockTriggered of the %d Channels are shown\n\n",unit->channelCount);
			fprintf(fp, "Time Voltage\n");
			fprintf(	fp,	"ns		mV\n");
			for(i = 0; i < nSamples; i++)
			{
				iSumm=0;
				if(unit->channelSettings[channel=0].enabled)
				for (capture = 0; capture < nCaptures; capture++) 
				iSumm=iSumm+rapidBuffers[0][capture][i];
				
				dAverage=(double)iSumm/(double)nCaptures;

				fprintf(fp, "%5lld ", g_times[0] + (long long)(i * timeInterval));

				dAverageVoltage=dAdc_to_mv(dAverage,unit->channelSettings[PS3000A_CHANNEL_A].range, unit);
				fprintf(fp, "   %lf       \n", dAverageVoltage);
			}
		}
		else
			printf(	"Cannot open the file rapidblock_average.txt for writing.\n"
			"Please ensure that you have permission to access.\n");


	if (fp != NULL)
		fclose(fp);



	//Free memory
	free(overflow);

	for (channel = 0; channel < unit->channelCount; channel++) 
	{	
		if(unit->channelSettings[channel].enabled)
		{
			for (capture = 0; capture < nCaptures; capture++) 
			{
				free(rapidBuffers[channel][capture]);
			}
		}
	}

	for (channel = 0; channel < unit->channelCount; channel++) 
	{
		free(rapidBuffers[channel]);
	}
	free(rapidBuffers);
}

/****************************************************************************
* Initialise unit' structure with Variant specific defaults
****************************************************************************/
void get_info(UNIT * unit)
{
	char description [6][25]= { "Driver Version",
		"USB Version",
		"Hardware Version",
		"Variant Info",
		"Serial",
		"Error Code" };
	short i, r = 0;
	char line [80];
	int variant;
	PICO_STATUS status = PICO_OK;

	if (unit->handle) 
	{
		for (i = 0; i < 5; i++) 
		{
			status = ps3000aGetUnitInfo(unit->handle, line, sizeof (line), &r, i);
			if (i == 3) 
			{
				variant = atoi(line);
			}
			printf("%s: %s\n", description[i], line);
		}
		unit->firstRange = PS3000A_50MV; 
		unit->lastRange = PS3000A_20V;
		unit->channelCount = 2;
	}
}

/****************************************************************************
* Select input voltage ranges for channels
****************************************************************************/
void SetVoltages(UNIT * unit)
{
	int i, ch;
	int count = 0;

	/* See what ranges are available... */
	for (i = unit->firstRange; i <= unit->lastRange; i++) // 
	{
		printf("%d -> %d mV\n", i, inputRanges[i]);
	}

	do
	{
		/* Ask the user to select a range */
		printf(">Please select the voltage range (%d..%d)\n", unit->firstRange, unit->lastRange);
		printf("99 - switches channel off\n");
		for (ch = 0; ch < unit->channelCount; ch++) 
		{
			printf("\n");
			do 
			{
				printf("The Channel %c: ", 'A' + ch);
				fflush(stdin);
				scanf_s("%hd", &unit->channelSettings[ch].range);
			} while (unit->channelSettings[ch].range != 99 && (unit->channelSettings[ch].range < unit->firstRange || unit->channelSettings[ch].range > unit->lastRange));

			if (unit->channelSettings[ch].range != 99) 
			{
				printf(" - %d mV\n", inputRanges[unit->channelSettings[ch].range]);
				unit->channelSettings[ch].enabled = TRUE;
				count++;
			} 
			else 
			{
				printf("The Channel Switched off\n");
				unit->channelSettings[ch].enabled = FALSE;
			}
		}
		printf(count == 0? "\n** At least 1 channel must be enabled **\n\n":"");
	}
	while(count == 0);	// must have at least one channel enabled

	SetDefaults(unit);	// Put these changes into effect
}

/****************************************************************************
*
* Select timebase, set oversample to on and time units as nano seconds
*
****************************************************************************/
void SetTimebase(UNIT unit)
{
	long timeInterval;
	long maxSamples;

	printf("0 -> 2 ns, 1 -> 4 ns, 2 -> 8 ns, 3 -> 16 ns, 4 -> 32 ns, ... max 2^32-1 -> ~ 68.7 s.\n ");

	printf("Specify desired timebase: ");
	fflush(stdin);
	scanf_s("%lud", &timebase);

	while (ps3000aGetTimebase(unit.handle, timebase, mySetting.nBufferSize, &timeInterval, 1, &maxSamples, 0))
	{
		timebase++;  // Increase timebase if the one specified can't be used. 
	}

	printf("Timebase used %lu = %ldns Sample Interval\n", timebase, timeInterval);
	oversample = TRUE;
}


/****************************************************************************
* CollectRapidNBlock
*  The Maximum number of frames in memory is 25000  (thus RapidBlock limited by this value). 25000 frames will named "a block".
*  This new function demonstrated how to collect N blocks (each from 25000 frames). 
*  The Averaged frame from each block are collected into a new super-block, which averaged too.
*  Therefore defaulr settings are 1 block = 25000 frames, 1 frames = 1024 samples.
****************************************************************************/
void CollectRapidNBlock(UNIT * unit)
{
	unsigned short nCaptures = mySetting.nWaveForms; // max 25000 at 1024 points;
	unsigned long nMaxSamples, nSamples = mySetting.nBufferSize;
	FILE * fp = NULL;
	long timeIndisposed;
	short capture, channel;
	short ***rapidBuffers;
	short *overflow;
	PICO_STATUS status;
	unsigned short i; // counter for frames inside a block
	unsigned int j; // counter for blocks
	unsigned long nCompletedCaptures;
	unsigned long nBlocks; // for write after user asking

	double *dNBlockAveraged; // for dinamic array for collect data

	
	short	triggerVoltage = mv_to_adc(mySetting.nTriggerLevel, unit->channelSettings[PS3000A_CHANNEL_A].range, unit);

	// PS3000A_TRIGGER_CHANNEL_PROPERTIES structure, see the manual, p.82
	struct tPS3000ATriggerChannelProperties sourceDetails = {	triggerVoltage,
																256 * 10,
																triggerVoltage,
																256 * 10,
																PS3000A_EXTERNAL, 
																PS3000A_LEVEL};
	// PS3000A_TRIGGER_CONDITIONS structure, see the manual, p. 79. 
	struct tPS3000ATriggerConditions conditions = {	PS3000A_CONDITION_DONT_CARE, // channelA
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_TRUE, // external input
													PS3000A_CONDITION_DONT_CARE,
													PS3000A_CONDITION_DONT_CARE};

	struct tPwq pulseWidth;

	struct tTriggerDirections directions = {	PS3000A_NONE, // channelA
												PS3000A_NONE,
												PS3000A_NONE,
												PS3000A_NONE,
												mySetting.bEdgeSelect, // channelEXT
												PS3000A_NONE };




	// These for ps3000aGetTimebase()
	long sampleCount; 
	long timeInterval;
	long maxSamples;

	sampleCount= nSamples;
	while (ps3000aGetTimebase(unit->handle, timebase, sampleCount, &timeInterval, oversample, &maxSamples, 0))
	{
		timebase++;
	}

	// Request the number of required blocks (1 block = 25000 frames, 1 frames = 1024 samples)
	printf("\n");
	printf("Enter the number of blocks (each block is %lu frames): ", (unsigned long)mySetting.nWaveForms); 
	fflush(stdin);
	scanf_s("%lud", &nBlocks);
//	printf("Collecting %lu blocks\n", nBlocks);


	//  Create array for collected data. Averaging is at the end of collecting (by divide of collected data to the number of cycles). 
	dNBlockAveraged=(double*)calloc(nSamples,sizeof(*dNBlockAveraged));
	if(dNBlockAveraged==0)
	{
		printf("Memory for dNBlockAveraged hasn't been allocated.");
		printf("\nPress any key...\n\n");
		_getch();		
		exit(0);
	}

	for(j=0;j<nBlocks; j++) // It is bad idea to allocate the memory by cycles. One must be allocated one time and be in use. But I couldn't have done it....
	{
		
		memset(&pulseWidth, 0, sizeof(struct tPwq));

		if(j==0) // Logs' output on screen for first block only.
		{
			printf("Collects will be started when voltage rises past %d",	scaleVoltages?
				adc_to_mv(sourceDetails.thresholdUpper, unit->channelSettings[PS3000A_CHANNEL_A].range, unit)	// If scaleVoltages, print value in mV
				: sourceDetails.thresholdUpper);																// else print ADC Count
			printf(scaleVoltages?"mV\n" : "ADC Counts\n");
			printf("Press any key to abort\n");
		}

		SetDefaults(unit);
	
		// Trigger enabled
		if ((status = SetTrigger(unit->handle, &sourceDetails, 1, &conditions, 1, &directions, &pulseWidth, 0, 0, 0)) != PICO_OK) 
		{
			printf("CollectRapidBlock:SetTrigger ------ Ox%8lx \n", status);
			return;
		}

		//Segment the memory
		if ((status = ps3000aMemorySegments(unit->handle, nCaptures, &nMaxSamples)) != PICO_OK) 
		{
			printf("CollectRapidBlock:MemorySegments ------ Ox%8lx \n", status);
			return;
		}

		//Set the number of captures
		if ((status = ps3000aSetNoOfCaptures(unit->handle, nCaptures)) != PICO_OK) 
		{
			printf("CollectRapidBlock:SetNoOfCaptures ------ Ox%8lx \n", status);
			return;
		}

		//Run
		if ((ps3000aRunBlock(unit->handle, 0, nSamples, timebase, 1, &timeIndisposed, 0, CallBackBlock, NULL)) != PICO_OK) 
		{
			printf("CollectRapidBlock:RunBlock ------ Ox%8lx \n", status);
			return;
		}


		g_ready = FALSE; 
		if(j==0)
		printf("Waiting for the trigger... ");
		while(!g_ready && !_kbhit())
		{
			Sleep(0);
		}

		if(!g_ready)
		{
			_getch();
			status = ps3000aStop(unit->handle);
			status = ps3000aGetNoOfCaptures(unit->handle, &nCompletedCaptures);
			printf("Rapid capture aborted. %lu complete blocks have been captured\n", nCompletedCaptures);
			printf("\nPress any key...\n\n");
			_getch();

			if(nCompletedCaptures == 0)
				return;

			//Only display the blocks that were captured
			nCaptures = (unsigned short)nCompletedCaptures;
		}

		if(j==0)
		printf("Done\n");
		//Allocate memory
		rapidBuffers = calloc(unit->channelCount, sizeof(short*));
		overflow = calloc(unit->channelCount * nCaptures, sizeof(short));

		for (channel = 0; channel < unit->channelCount; channel++) 
		{
			rapidBuffers[channel] = calloc(nCaptures, sizeof(short*));
		}

		for (channel = 0; channel < unit->channelCount; channel++) 
		{	
			if(unit->channelSettings[channel].enabled)
			{
				for (capture = 0; capture < nCaptures; capture++) 
				{
					rapidBuffers[channel][capture] = calloc(nSamples, sizeof(short));
				}
			}
		}

		if(j==0)
		printf("Collecting data...\n");
		for (channel = 0; channel < unit->channelCount; channel++) 
		{
			if(unit->channelSettings[channel].enabled)
			{
				for (capture = 0; capture < nCaptures; capture++) 
				{
					status = ps3000aSetDataBuffer(unit->handle, channel, rapidBuffers[channel][capture], nSamples, capture, PS3000A_RATIO_MODE_NONE);
				}
			}
		}

		//Get data
		status = ps3000aGetValuesBulk(unit->handle, &nSamples, 0, nCaptures - 1, 1, PS3000A_RATIO_MODE_NONE, overflow);

		//Stop
		status = ps3000aStop(unit->handle);

		// Averaging frames from a block
		if(j==0) // Write the first block data to the file for demo or testing.
		{
			printf("Averaged %hu captures      \n", nCaptures);
			fopen_s(&fp, "rapidblock_average.txt", "w");
		}

			if ((fp != NULL)||(j!=0)) // 
			{
				double dAverage;
				long iSumm;
				double dAverageVoltage;

				if(j==0)
				{
					fprintf(fp,"Results shown for CollectRapidBlockTriggered of the %d Channels\n\n",unit->channelCount);
					fprintf(fp, "Time Voltage\n");
					fprintf(	fp,	"ns		mV\n");
				}
				for(i = 0; i < nSamples; i++)
				{
					iSumm=0;
					if(unit->channelSettings[channel=0].enabled)
					for (capture = 0; capture < nCaptures; capture++) 
					iSumm=iSumm+rapidBuffers[0][capture][i];
				
					dAverage=(double)iSumm/(double)nCaptures;

					if(j==0)
					fprintf(fp, "%5lld ", g_times[0] + (long long)(i * timeInterval));

					// to average ADC points, and convert them to mV.
					dAverageVoltage=dAdc_to_mv(dAverage,unit->channelSettings[PS3000A_CHANNEL_A].range, unit);
					if(j==0)
					fprintf(fp, "   %lf       \n", dAverageVoltage);

					// Put the averaged array into accumulator
					dNBlockAveraged[i]=dNBlockAveraged[i]+dAverageVoltage;

				}
			}
			else
				printf(	"Cannot open the file rapidblock_average.txt for writing.\n"
				"Please ensure that you have permission to access.\n");

		if(j==0)
		if (fp != NULL)
			fclose(fp);




		//Free memory
		free(overflow);

		for (channel = 0; channel < unit->channelCount; channel++) 
		{	
			if(unit->channelSettings[channel].enabled)
			{
				for (capture = 0; capture < nCaptures; capture++) 
				{
					free(rapidBuffers[channel][capture]);
				}
			}
		}

		for (channel = 0; channel < unit->channelCount; channel++) 
		{
			free(rapidBuffers[channel]);
		}
		free(rapidBuffers);

		printf("\r"); // Go to the start of the string
		printf("Averaged %lu blocks from %lu (total %ld captures)", j+1, nBlocks, (long)nBlocks*(long)nCaptures); 
	

	}
	printf("\n");
	// Divide accumulated data on current cycles number.
	for(i=0;i<nSamples; i++)
	dNBlockAveraged[i]=dNBlockAveraged[i]/(double)j;

	// File for write of accumulated and averaged array.
	printf("Write %hu averaged blocks  (total %ld averages)\n ", j, (long)j*(long)nCaptures);

	fopen_s(&fp, cFileName, "w");

	if (fp != NULL)
	{
			fprintf(fp,"Results are shown for CollectRapidNBlock for the %d Channels, %ld averages\n\n",unit->channelCount, (long)j*(long)nCaptures);
			fprintf(fp, "Time Voltage\n");
			fprintf(	fp,	"ns		mV\n");
			for(i = 0; i < nSamples; i++)
			{
				fprintf(fp, "%5lld ", g_times[0] + (long long)(i * timeInterval));
				fprintf(fp, "   %lf       \n", dNBlockAveraged[i]);
			}
	}
	else
		printf(	"Cannot open the file for writing N block.\n"
				"Please ensure that you have permission to access.\n");

	if (fp != NULL)
			fclose(fp);

	free(dNBlockAveraged);

}



/****************************************************************************
* DisplaySettings 
* Displays information about the user configurable settings in this example
* Parameters 
* - unit        pointer to the UNIT structure
*
* Returns       none
***************************************************************************/
void DisplaySettings(UNIT *unit)
{
	int ch;
	int voltage;

	//=== for display of SampleInterval. Peter.
	long timeInterval;
	long sampleCount= mySetting.nBufferSize;
	FILE * fp = NULL;
	long maxSamples;


	printf("\nReadings will be scaled in (%s)\n", (scaleVoltages)? ("mV") : ("ADC counts"));

	for (ch = 0; ch < unit->channelCount; ch++)
	{
		voltage = inputRanges[unit->channelSettings[ch].range];
		printf("Channel %c Voltage Range = ", 'A' + ch);

		//if (voltage == 0)
		if(unit->channelSettings[ch].enabled != TRUE)
			printf("Off\n");
		else
		{
			if (voltage < 1000)
				printf("%dmV\n", voltage);
			else
				printf("%dV\n", voltage / 1000);
		}
	}

	// Needed to add on/off channel 
	while (ps3000aGetTimebase(unit->handle, timebase, sampleCount, &timeInterval, oversample, &maxSamples, 0))
	{
		timebase++;
	}
	printf("Timebase = %lu, SampleInterval: %ldns, Trigger = %d mV.\n", timebase, timeInterval, mySetting.nTriggerLevel); 

	printf("\n");

}

/****************************************************************************
* OpenDevice 
* Parameters 
* - unit        pointer to the UNIT structure, where the handle will be stored
*
* Returns
* - PICO_STATUS to indicate success, or if an error occurred
*
* ADDITIONS:
* Reading setting from the file (or function load default setting and create new setting file).
***************************************************************************/
PICO_STATUS OpenDevice(UNIT *unit) 
{
	short value = 0;
	int i;
	struct tPwq pulseWidth;
	struct tTriggerDirections directions;
	PICO_STATUS status = ps3000aOpenUnit(&(unit->handle), NULL);


	// for setting from and into the file
	FILE * fset = NULL;
	int buf; // for clean the string buffer
	char buffer[100];



	printf("Handle: %d\n", unit->handle);
	if (status != PICO_OK) 
	{
		printf("Unable to open device\n");
		printf("Error code : 0x%08lx\n", status);
		while(!_kbhit());
		exit(99); // exit program
	}

	printf("Device opened successfully\n");

	// setup devices

	fopen_s(&fset, cSettingFile, "r");

	// If setting file exist, setting are loaded from its. 
	// If setting file don't exist, setting are loaded by default and the file are created. 
	// (It is better to create a separate function for set parameters. It should let us run "return()" without the next cycle.)

	// File exist. Read setting from the file to the structure
	if (fset != NULL) 
	{
		
		printf(	"Reading setting from ");
		printf(cSettingFile, sizeof(cSettingFile));
		printf(	".\n");

		// Read a string from file to the buffer, convert buffer string to a variable, save the variable.
		fscanf(fset, "%s",buffer);
		fscanf(fset, "%s",buffer);
		if(buffer[0]==0)
		mySetting.percTimeShift=atoi(buffer+1);// +1 - To shift the start, else will be read 00. Because I had a problem.
		else
		mySetting.percTimeShift=atoi(buffer);

		fscanf(fset, "%s",buffer);
		fscanf(fset, "%s",buffer);
		if(buffer[0]==0)
		mySetting.nBufferSize=atoi(buffer+1);// +1 - To shift the start, else will be read 00.
		else
		mySetting.nBufferSize=atoi(buffer);
		printf(	"percTimeShift=%d, nBufferSize=%d, ",mySetting.percTimeShift, mySetting.nBufferSize);

		fscanf(fset, "%s",buffer);
		fscanf(fset, "%s",buffer);
		if(buffer[0]==0)
		mySetting.nTriggerLevel=atoi(buffer+1);// +1 - To shift the start, else will be read 00
		else
		mySetting.nTriggerLevel=atoi(buffer);

		fscanf(fset, "%s",buffer);
		fscanf(fset, "%s",buffer);
		if(buffer[0]==0)
		mySetting.bEdgeSelect=atoi(buffer+1);// +1 - To shift the start, else will be read 00
		else
		mySetting.bEdgeSelect=atoi(buffer);

		printf(	"nTriggerLevel=%d, bEdgeSelect=%d,\n",mySetting.nTriggerLevel, (int)mySetting.bEdgeSelect);
		
		fscanf(fset, "%s",buffer);
		fscanf(fset, "%s",buffer);
		if(buffer[0]==0)
		mySetting.nWaveForms=atoi(buffer+1);// +1 - To shift the start, else will be read 00
		else
		mySetting.nWaveForms=atoi(buffer);

		fscanf(fset, "%s",buffer);
		fscanf(fset, "%s",buffer);
		if(buffer[0]==0)
		mySetting.timeBase=atol(buffer+1);// +1 - To shift the start, else will be read 00
		else
		mySetting.timeBase=atol(buffer);

		printf(	"nWaveForms=%d, timeBase=%d, ",mySetting.nWaveForms, mySetting.timeBase);
		
		fscanf(fset, "%s",buffer);
		fscanf(fset, "%s",buffer);
		if(buffer[0]==0)
		mySetting.channelRange=atoi(buffer+1);// +1 - To shift the start, else will be read 00
		else
		mySetting.channelRange=atoi(buffer);

		fscanf(fset, "%s",buffer);
		fscanf(fset, "%s",buffer);
		if(buffer[0]==0)
		mySetting.channelDCcoupled=atoi(buffer+1);// +1 - To shift the start, else will be read 00
		else
		mySetting.channelDCcoupled=atoi(buffer);

		printf(	"channelRange=%d, channelDCcoupled=%d,\n",mySetting.channelRange, mySetting.channelDCcoupled);
		
		fscanf(fset, "%s",buffer);
		fscanf(fset, "%s",buffer);
		if(buffer[0]==0)
		mySetting.switchChannelA=atoi(buffer+1);// +1 - To shift the start, else will be read 00
		else
		mySetting.switchChannelA=atoi(buffer);

		fscanf(fset, "%s",buffer);
		fscanf(fset, "%s",buffer);
		if(buffer[0]==0)
		mySetting.switchChannelB=atoi(buffer+1);// +1 - To shift the start, else will be read 00
		else
		mySetting.switchChannelB=atoi(buffer);

		printf(	"switchChannelA=%d, switchChannelB=%d.\n",mySetting.switchChannelA, mySetting.switchChannelB);


	fclose(fset);


	}
	else
	{

		// Load default setting and save them to file

		// initialization of structure, these are DEFAULT setting: 
		mySetting.percTimeShift=10;
		mySetting.nBufferSize=1024;
		mySetting.nTriggerLevel=-20; //100
		mySetting.bEdgeSelect=PS3000A_FALLING;
		mySetting.nWaveForms=25000;
		mySetting.timeBase = 1;  // 0 - 2ns, 1 - 4ns, 2 - 8ns, 3 - 16 ns, 4 - 32ns.
		mySetting.channelRange=PS3000A_500MV; // PS3000A_50MV, ...100,200,500..., PS3000A_1V, ...5,10...PS3000A_20V
		mySetting.channelDCcoupled=TRUE;
		mySetting.switchChannelA=TRUE;
		mySetting.switchChannelB=FALSE;
		
	
		fopen_s(&fset, cSettingFile, "w");
		if (fset != NULL)
		{
			printf(	"Default setting loaded into ");
			printf(cSettingFile, sizeof(cSettingFile));
			printf(	".\n");

			// Write "Parameter" and "Value", string by string.
			for(buf=0; buf<100;buf++)
			buffer[buf]=32; // hex 20, space sign.

			fwrite("[Time_shift_in_percent]\n", sizeof("[Time_shift_in_percent]\n"),1, fset);
			_itoa(mySetting.percTimeShift,buffer, 10);// It add "00" before a number (I don't know why). Therefore I use "+1" in read setting from file.
			fputs (buffer, fset);
			fputs ("\n", fset);

			fwrite("[Record_lenght_in_bytes]\n", sizeof("[Record_lenght_in_bytes]\n"),1, fset); 
			_itoa(mySetting.nBufferSize,buffer, 10);
			fputs (buffer, fset);
			fputs ("\n", fset);

			fwrite("[Triggel_level_in_mV]\n", sizeof("[Triggel_level_in_mV]\n"),1, fset); 
			_itoa(mySetting.nTriggerLevel,buffer, 10);
			fputs (buffer, fset);
			fputs ("\n", fset);

			fwrite("[Trigger_edge_select:0-above_1-below_2-rising_3-falling]\n", sizeof("[Trigger_edge_select:0-above_1-below_2-rising_3-falling]\n"),1, fset); 
			_itoa(mySetting.bEdgeSelect,buffer, 10);
			fputs (buffer, fset);
			fputs ("\n", fset);

			fwrite("[Number_of_waveforms_in_the_block]\n", sizeof("[Number_of_waveforms_in_the_block]\n"),1, fset); 
			_itoa(mySetting.nWaveForms,buffer, 10);
			fputs (buffer, fset);
			fputs ("\n", fset);

			fwrite("[Time_scale:0-2ns_1-4ns_2-8ns_3-16ns_etc]\n", sizeof("[Time_scale:0-2ns_1-4ns_2-8ns_3-16ns_etc]\n"),1, fset); 
			_itoa(mySetting.timeBase,buffer, 10);
			fputs (buffer, fset);
			fputs ("\n", fset);

			fwrite("[Channel_Sentivity:2-50mV_3-100mV_4-200mV_5-500mV_etc]\n", sizeof("[Channel_Sentivity:2-50mV_3-100mV_4-200mV_5-500mV_etc]\n"),1, fset); 
			_itoa(mySetting.channelRange,buffer, 10);
			fputs (buffer, fset);
			fputs ("\n", fset);

			fwrite("[DC_mode:0-AC_1-DC]\n", sizeof("[DC_mode:0-AC_1-DC]\n"),1, fset); 
			_itoa(mySetting.channelDCcoupled,buffer, 10);
			fputs (buffer, fset);
			fputs ("\n", fset);

			fwrite("[Channel_A:0-disabled_1-enabled]\n", sizeof("[Channel_A:0-disabled_1-enabled]\n"),1, fset); 
			_itoa(mySetting.switchChannelA,buffer, 10);
			fputs (buffer, fset);
			fputs ("\n", fset);

			fwrite("[Channel_B:0-disabled_1-enabled]\n", sizeof("[Channel_B:0-disabled_1-enabled]\n"),1, fset); 
			_itoa(mySetting.switchChannelB,buffer, 10);
			fputs (buffer, fset);
			fputs ("\n", fset);

			fputs ("\n\n", fset);
			fwrite("NB! Don't change the file's structure! You should just delete this file if you need to restore setting. It will be recovered here.\n", sizeof("NB! Don't change the file's structure! You should just delete this file if you need to restore setting. It will be recovered here.\n"),1, fset); 

			fclose(fset);
		}
		else
		printf(	"Cannot open the file for setting.\n");
		if (fset != NULL)
			fclose(fset);
	}
	
	get_info(unit);
	timebase = mySetting.timeBase;

	ps3000aMaximumValue(unit->handle, &value);
	unit->maxValue = value;

	for ( i = 0; i < PS3000A_MAX_CHANNELS; i++) 
	{
		unit->channelSettings[i].DCcoupled = TRUE;
		unit->channelSettings[i].range = PS3000A_500MV; 
	}
		unit->channelSettings[0].enabled = TRUE; // chA enabled
		unit->channelSettings[1].enabled = FALSE; // chB disabled


	memset(&directions, 0, sizeof(struct tTriggerDirections));
	memset(&pulseWidth, 0, sizeof(struct tPwq));

	SetDefaults(unit);

	/* Trigger disabled	*/ // I must enabled its to swithc to EXT.
	SetTrigger(unit->handle, NULL, 0, NULL, 0, &directions, &pulseWidth, 0, 0, 0);

	return status;
}


/****************************************************************************
* CloseDevice 
****************************************************************************/
void CloseDevice(UNIT *unit)
{
	ps3000aCloseUnit(unit->handle);
}

/****************************************************************************
* main
* 
***************************************************************************/
int main(void)
{
	char ch;
	UNIT unit;
	PICO_STATUS status;



	printf("Picoscope averager for PS3000A. Version 1.0.\n");
	printf("Opening the device...\n");



	status = OpenDevice(&unit);


	ch = '.';
	while (ch != 'X')
	{
		DisplaySettings(&unit);

		printf("\n");
		printf("S - Single sample, no average           V - Set voltages\n");
		printf("A - Average of %ld samples            T - Set timebase\n",mySetting.nWaveForms);
		printf("N - Average of Nx%ld samples          U - ADC counts/mV\n",mySetting.nWaveForms);
		printf("                                        X - Exit\n");
		printf("Operation:");

		ch = toupper(_getch());

//		printf("\n\n");
		switch (ch) 
		{
		case 'S':
			CollectBlockTriggered(&unit); 
			break;

		case 'A':
			CollectRapidBlock(&unit);
			break;

		case 'N':
			CollectRapidNBlock(&unit);
			break;

		case 'V':
			SetVoltages(&unit);
			break;

		case 'T':
			SetTimebase(unit);
			break;

		case 'U':
			scaleVoltages = !scaleVoltages;
			break;

		case 'X':
			break;

		default:
			printf("Invalid operation\n");
			break;
		}
	}
	CloseDevice(&unit);

	return 1;
}
